/*#######################################################################################

AD-Wandler
Register-Namen mssen ggf. in "adc.c" angepasst werden

Copyright (C) 2013 Leo-Andres Hofmann
www.leo-andres.de

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.

#######################################################################################*/

#ifndef _ADC_H_
	#define _ADC_H_

////////////////////////////////////////////////////////////////////////////////////////////
//Einstellungen

//Vorteiler fr den Takt des Wandlers, Zielfrequenz 50-200kHz (128 fr ~120kHz @ ~15MHz F_CPU)
#define ADC_PRESCALER		(1<<ADPS2) | (1<<ADPS1) | (1<<ADPS0)

//Referenzspannung & Kanal
#define ADC_REFERENCE		(1<<REFS1) | (1<<REFS0)

//Anzahl Samples pro Messung, ber die ein Mittelwert gebildet wird (max. 60)
#define ADC_SAMPLES			32

//Laufzeitkalibrierung aktivieren
//Diese Funktion behebt Ungenauigkeiten, wenn die Betriebsspannung als Referenz verwendet wird
//Die interne Referenzspannung wird gegen die Betriebsspannung gemessen und ein Korrekturfaktor errechnet
#define ADC_RUNTIME_CALIBRATION		FALSE

//Interne Referenzspannung in mV
#define ADC_AREF_MV			1100

//MUX-Bits fr interne Referenz
#define ADC_MUX_REFERENCE	0xE

////////////////////////////////////////////////////////////////////////////////////////////

//AD-Wandler initialisieren
extern void adc_init(void);

//Einen AD-Kanal lesen
extern uint16_t adc_read(uint8_t channel);

//Einen AD-Kanal lesen und einen Mittelwert bilden
extern uint16_t adc_readAverage(uint8_t channel);

//Gibt einen Spannungswert in mV zurck, Mittelwert-Messung
//Die Spannung wird mit der laufzeitkalibrierten bzw. definierten Referenzspannung ermittelt
extern uint16_t adc_readVoltage(uint8_t Channel);

#if ADC_RUNTIME_CALIBRATION == TRUE

//Laufzeitkalibrierung auslsen
extern void adc_runtimeCalibrate(void);

//Gibt die laufzeitkalibrierte Referenzspannung in mV zurck
extern uint16_t adc_getRuntimeReference(void);

#endif

#endif //_ADC_H_
