/*#######################################################################################

Allgemeine Funktionen, globale Statusvariablen

Arbeitet eng mit main.c und ports.h zusammen, eigenstndig kaum benutzbar


Copyright (C) 2013 Leo-Andres Hofmann
www.leo-andres.de

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.

#######################################################################################*/

#ifndef _GENERAL_H_
	#define _GENERAL_H_

#include <avr/pgmspace.h>

#include "settings.h"

////////////////////////////////////////////////////////////////////////////////////////////
//Einstellungen (aus "settings.h" bernommen!)

//Systemtakt: Entweder aus dem Makefile, oder hier angeben
#ifndef SYSCLOCK
	#ifdef F_CPU
		#define SYSCLOCK F_CPU
	#else
		#define SYSCLOCK 14756000UL	//Quarz-Frequenz in Hz
		#define F_CPU SYSCLOCK
	#endif
#endif

//Watchdog-Timeout
//Mgliche Einstellungen: 15MS, 30MS, 60MS, 120MS, 250MS, 500MS, 1S, 2S
#ifndef WATCHDOG_TIMEOUT
	#define WATCHDOG_TIMEOUT	WDTO_2S
#endif

////////////////////////////////////////////////////////////////////////////////////////////
//Event-Timer

//Prescaler Event-Timer
#define GENERAL_EVENTS_PRESCALER	(1 << CS02) | (1 << CS00)	//1024

//Timer-Overflow-Interrupt aktivieren
#define GENERAL_EVENTS_ENABLE		TIMSK0 = (1<<TOIE0)	

//Kontrollregister des Event-Timers
#define GENERAL_EVENTS_CONTROL		TCCR0B

//Interrupt-Vektor
#define GENERAL_EVENTS_INT			TIMER0_OVF_vect

//Bentigte Anzahl Events (maximal 8!)
#define GENERAL_EVENTS_COUNT		3

//Trigger-Bits fr einzelne Events
//Die Bits werden bei Erreichen eines Events in der Variable "General_EventsTrigger" gesetzt
#define GENERAL_EVENTS_TRIGGER_1	0x01
#define GENERAL_EVENTS_TRIGGER_2	0x02
#define GENERAL_EVENTS_TRIGGER_3	0x04
#define GENERAL_EVENTS_TRIGGER_4	0x08
#define GENERAL_EVENTS_TRIGGER_5	0x10
#define GENERAL_EVENTS_TRIGGER_6	0x20
#define GENERAL_EVENTS_TRIGGER_7	0x40
#define GENERAL_EVENTS_TRIGGER_8	0x80

//Software-Teiler der Events
static const PROGMEM uint8_t general_eventsSettings[GENERAL_EVENTS_COUNT] =
{
	3,		//Event 1
	6,		//Event 2
	15		//Event 3
};

//Globaler Speicher fr Trigger
extern volatile uint8_t general_eventsTrigger;

//Hilfsmakros fr die Events
#define general_eventTriggered(event) (general_eventsTrigger & event)
#define general_resetTrigger(trigger) general_eventsTrigger &= ~(trigger);

////////////////////////////////////////////////////////////////////////////////////////////
//Allgemeine Definitionen

//Status-Variablen
#define	TRUE	1
#define	FALSE	0

#define OFF		0	//Spannungen
#define ON		1
#define BAD		2

#define MCU		1	//UART-Link
#define STK		2

////////////////////////////////////////////////////////////////////////////////////////////
//Allgemeine Funktionen

//1 Takt nop
#define nop() __asm volatile ("nop")

////////////////////////////////////////////////////////////////////////////////////////////
//Status-Register

struct general_status
{
	unsigned resetByWDT:		1;	//Controller wurde vom Watchdog zurckgesetzt
	unsigned resetWDT:			1;	//Watchdog-Timer weiterhin zurcksetzen (damit lsst sich ein Reset auslsen)

	unsigned ledToggle:			1;	//Hilfsbits fr LED-Anzeige
	unsigned ledFastToggle:		1;

	unsigned uartSW_State:		2;	//Zwischenspeicher Tastenentprellung
	unsigned uartSW_Signal:		1;

	unsigned resetSW_State:		2;
	unsigned resetSW_Signal:	1;
	unsigned resetSW_Active:	1;

	unsigned VCC_USB:			2;	//Status der Versorgungsspannungen
	unsigned VCC_USB_Overload:	1;

	unsigned VCC_MCU:			2;

	unsigned UART_Link:			2;
};

//Globales Statusregister
extern struct general_status general_status;

////////////////////////////////////////////////////////////////////////////////////////////
//Statuscodes Controller-Reset

#define GENERAL_RESET_BROWN_OUT		1	//Brown-Out Reset (Spannungseinbrucht)
#define GENERAL_RESET_POWER_ON		2	//Power-On Reset (Einschalten)
#define GENERAL_RESET_EXTERN		3	//Externer Reset (Programmiergert, Reset-Schaltung)
#define GENERAL_RESET_WATCHDOG		4	//Watchdog-Timer
#define GENERAL_RESET_UNKNOWN		5	//Unbekannt

////////////////////////////////////////////////////////////////////////////////////////////
//Funktionen

//Initialisiert den Timer fr das Event-System
extern void general_initEvents(void);

//Watchdog initialisieren
//Das Timeout wird ber "WATCHDOG_TIMEOUT" eingestellt
extern void general_initWatchdog(void);

//Watchdog aktivieren
extern void general_enableWatchdog(void);

//Watchdog deaktivieren
extern void general_disableWatchdog(void);

//Reset-Status des Controllers auslesen
//Rckgabe siehe "Statuscodes Controller-Reset"
extern uint8_t general_readResetState(void);

//Statusregister auf Startwerte setzen
extern void general_initStatus(void);

//Mikrocontroller per Watchdog neu starten
extern void general_reboot(void);

#endif //_GENERAL_H_
