/*#######################################################################################

Funktionen zum Einstellen der I/O-Ports

Arbeitet eng mit main.c und general.h zusammen, eigenstndig kaum benutzbar


Copyright (C) 2013 Leo-Andres Hofmann
www.leo-andres.de

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.

#######################################################################################*/

#ifndef _PORTS_H_
	#define _PORTS_H_

////////////////////////////////////////////////////////////////////////////////////////////
//Unbenutze Pins & Ports (Pullups aktivieren)

//Port 1: PORTC
#define FREE_1_PORT		PORTC
#define FREE_1_PINS		(1<<PC2)

////////////////////////////////////////////////////////////////////////////////////////////
//Bus

//UART
#define UART_RX_PIN		PD0
#define UART_TX_PIN		PD1
#define UART_PORT		PORTD

//TWI
#define TWI_SDA_PIN		PC4
#define TWI_SCL_PIN		PC5
#define TWI_PORT		PORTC

//ISP-Master inkl. Reset
//Pin-Richtung wird vom Programmiersystem umgeschaltet!
#define ISP_MISO_PIN	PB4
#define ISP_MOSI_PIN	PB3
#define ISP_SCK_PIN		PB5
#define ISP_RESET_PIN	PB2
#define ISP_PORT		PORTB

////////////////////////////////////////////////////////////////////////////////////////////
//Eingnge

//berstrom-Meldung vom USB-Port
//teilt sich einen Pin mit der Fehler-LED, High-Pegel nur ber Tristate & Pullup!
//(Open-Collector Ausgang parallel zum Pin)
#define USB_FAULT_PIN	PC3
#define USB_FAULT_PORT	PORTC
#define USB_FAULT_DATA	PINC

//UART DTR
#define UART_DTR_PIN	PD2
#define UART_DTR_PORT	PORTD
#define UART_DTR_DATA	PIND

//Taster
#define SW_UART_PIN		PD7
#define SW_UART_PORT	PORTD
#define SW_UART_DATA	PIND

#define SW_RESET_PIN	PB0
#define SW_RESET_PORT	PORTB
#define SW_RESET_DATA	PINB

//AD-Eingnge (Spannungsmessung)
#define ADC_VCC_PIN		6	//MCU VCC
#define ADC_VCCIO_PIN	7	//USB VCC

////////////////////////////////////////////////////////////////////////////////////////////
//Makros fr Eingnge

#define USB_Fault_isInput	!(DDR(USB_FAULT_PORT) & (1<<USB_FAULT_PIN))

////////////////////////////////////////////////////////////////////////////////////////////
//Ausgnge

//UART-Konfiguration
#define UART_EN_PIN			PD3
#define UART_EN_PORT		PORTD

#define UART_SEL_PIN		PD4
#define UART_SEL_PORT		PORTD

//Taktsignal
#define CLK_PIN				PB1
#define CLK_PORT			PORTB

////////////////////////////////////////////////////////////////////////////////////////////
//Makros fr Ausgnge

#define UART_Enable			UART_EN_PORT &= ~(1<<UART_EN_PIN);
#define UART_Disable		UART_EN_PORT |= (1<<UART_EN_PIN);

#define UART_Target_MCU		UART_SEL_PORT |= (1<<UART_SEL_PIN);
#define UART_Target_STK		UART_SEL_PORT &= ~(1<<UART_SEL_PIN);

#define Clock_Off			{DDR(CLK_PORT) &= ~(1<<CLK_PIN); CLK_PORT &= ~(1<<CLK_PIN);}
#define Clock_On			{DDR(CLK_PORT) |= (1<<CLK_PIN); CLK_PORT &= ~(1<<CLK_PIN);}

////////////////////////////////////////////////////////////////////////////////////////////
//LEDs

//LED 1: Fehler, siehe USB_FAULT
#define LED_FAULT_PIN			PC3
#define LED_FAULT_PORT			PORTC

//LED 2 & 3: UART STK / MCU
#define LED_UART_STK_PIN		PD6
#define LED_UART_MCU_PIN		PD5
#define LED_UART_PORT			PORTD

//LED 4: Status
#define LED_STAT_RED_PIN		PC1
#define LED_STAT_GREEN_PIN		PC0
#define LED_STAT_PORT			PORTC

////////////////////////////////////////////////////////////////////////////////////////////
//Makros fr LEDs

//Fehler-LED: Pin wird auch als Eingang genutzt! High-Pegel nur ber Tristate!
#define LED_Fault_On 		{LED_FAULT_PORT &= ~(1<<LED_FAULT_PIN); DDR(LED_FAULT_PORT) |= (1<<LED_FAULT_PIN);}
#define LED_Fault_Off 		{DDR(LED_FAULT_PORT) &= ~(1<<LED_FAULT_PIN); LED_FAULT_PORT |= (1<<LED_FAULT_PIN);}

#define LED_Status_Off 		{LED_STAT_PORT |= (1<<LED_STAT_RED_PIN) | (1<<LED_STAT_GREEN_PIN);}
#define LED_Status_Red 		{LED_STAT_PORT &= ~(1<<LED_STAT_RED_PIN); LED_STAT_PORT |= (1<<LED_STAT_GREEN_PIN);}
#define LED_Status_Green 	{LED_STAT_PORT |= (1<<LED_STAT_RED_PIN); LED_STAT_PORT &= ~(1<<LED_STAT_GREEN_PIN);}
#define LED_Status_Orange 	{LED_STAT_PORT &= ~( (1<<LED_STAT_RED_PIN) | (1<<LED_STAT_GREEN_PIN) );}

#define LED_UART_Off 		{LED_UART_PORT |= (1<<LED_UART_STK_PIN) | (1<<LED_UART_MCU_PIN);}
#define LED_UART_STK 		{LED_UART_PORT &= ~(1<<LED_UART_STK_PIN); LED_UART_PORT |= (1<<LED_UART_MCU_PIN);}
#define LED_UART_MCU 		{LED_UART_PORT |= (1<<LED_UART_STK_PIN); LED_UART_PORT &= ~(1<<LED_UART_MCU_PIN);}
#define LED_UART_Both 		{LED_UART_PORT &= ~( (1<<LED_UART_STK_PIN) | (1<<LED_UART_MCU_PIN) );}

////////////////////////////////////////////////////////////////////////////////////////////

//Makro zur Bestimmung vom DDR
#define DDR(x) (*(&x - 1))    
#if defined(__AVR_ATmega64__) || defined(__AVR_ATmega128__)
    #define PIN(x) ( &PORTF==&(x) ? _SFR_IO8(0x00) : (*(&x - 2)) )
#else
	#define PIN(x) (*(&x - 2))   
#endif

////////////////////////////////////////////////////////////////////////////////////////////

//Initialisiert alle I/O-Ports
//Nach der Init sind alle Funktionen deaktiviert
extern void ports_init(void);

#endif //_PORTS_H_
