/*#######################################################################################

Globale Einstellungen

Hier werden die wichtigsten Einstellungen zusammengefasst.
Modul-spezifische Einstellungen (z.B. UART Stopbits) sind in den Bibliotheken definiert.


Copyright (C) 2013 Leo-Andres Hofmann
www.leo-andres.de

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.

#######################################################################################*/

#ifndef _SETTINGS_H_
	#define _SETTINGS_H_

#include <avr/pgmspace.h>

////////////////////////////////////////////////////////////////////////////////////////////
//Hardware

//Quarz-Frequenz in Hz
#define F_CPU 14745600UL

//Watchdog-Timeout
//Mgliche Einstellungen: 15MS, 30MS, 60MS, 120MS, 250MS, 500MS, 1S, 2S, 8S
#define WATCHDOG_TIMEOUT	WDTO_500MS

////////////////////////////////////////////////////////////////////////////////////////////
//Funktionssteuerung

//Minimale bzw. maximale Spannungen, Angabe in mV (hinter dem Spannungsteiler)
//USB- und MCU-Spannung wird nicht unterschieden da kein Pegelwandler vorhanden ist
#define VCC_MIN			745	//~4,25V
#define VCC_MAX			925	//~5,25V

//Schwellwert, unter dem eine Spannung als abgeschaltet interpretiert wird
#define VCC_OFF			265	//~1,5V

//berstrom-Meldung nur durch Neustart lschen
#define USB_OVERLOAD_LATCHING	TRUE

//Verzgerungszhler fr die LED-Anzeige
#define UART_LED_COUNTDOOWN		10
#define STK_LED_COUNTDOWN		5	//Reset-Pin wird gleichzeitig angesteuert

//Status-LED blinken nach Verbindungsaufbau
#define STK_LED_SIGNON_COUNTDOWN	25

//Nach dem Einschalten direkt mit dem Programmiersystem verbinden
#define UART_START_STK	TRUE

////////////////////////////////////////////////////////////////////////////////////////////
//STK500v2-Protokoll & Programmiersystem

//Fest definierte Parameter
#define STK_CONF_PARAM_TOPCARD				0xFF		//keine Erweiterungskarte = 0xFF
#define STK_CONF_PARAM_BUILD_NUMBER_LOW		0x00
#define STK_CONF_PARAM_BUILD_NUMBER_HIGH	0x01		//Software Build-Nummer
#define STK_CONF_PARAM_HW_VER				0x11		//Hardware-Version
#define STK_CONF_PARAM_SW_MAJOR				0x02
#define STK_CONF_PARAM_SW_MINOR				0x0A		//Software-Version

#define STK_CONF_SIGNATURE					"STK500_2"	//Board-Signatur, immer 8 Zeichen

//Watchdog auch im Programmiersystem zurcksetzen
#define STK_WATCHDOG_RESET		TRUE

//Anzahl Durchlufe fr RDY/BSY-Abfrage, danach Timeout
#define STK_RDYBSY_CYCLES		1000	//16 Bit
#define STK_POLLING_CYCLE		200		//8 Bit

//Startwerte fr Taktgeber, 250kHz
#define STK_CLOCK_DEFAULT_PRESCALER		1
#define STK_CLOCK_DEFAULT_MATCH			14

//Bootloader-Passwort fr FastBoot
//im Leerlauf wird auf das Passwort gewartet und der Bootloader gestartet
//(auskommentieren zum Deaktivieren)
#define BOOTLOADER_PASSWORD			"Leoa"	//keine STK500v2 Steuerzeichen!
#define BOOTLOADER_PASSWORD_LENGTH	4		//max. 8 Zeichen

//Impulslnge am Reset-Pin in ms, 0 = deaktiviert
#define RESET_PULSE_LENGTH		25

//Sperrzeit fr Reset-Interrupts, Timer-Ticks
//RESET_PULSE_IDLE >> STK_LED_COUNTDOWN!
#define RESET_PULSE_IDLE		25

////////////////////////////////////////////////////////////////////////////////////////////
//Schnittstellen

//UART-Baudrate
#define UART_BAUDRATE   115200UL

#endif //_SETTINGS_H_
