/*#######################################################################################

Programmiersystem

Fest programmierte Einstellungen siehe "STK_CONF_xyz" aus settings.h

Copyright (C) 2013 Leo-Andres Hofmann
www.leo-andres.de

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.

#######################################################################################*/

#ifndef _STK500_H_
	#define _STK500_H_

#include "stk500_tools.h"

////////////////////////////////////////////////////////////////////////////////////////////
//Einstellungen

//Pufferspeicher-Gre fr STK500v2-Nachrichten
#define STK_MSG_BUFFER		275

//RDY/BSY-Timeout Abfrage
#define STK_RDYBSY_ISP_CMD	0xF0000000

////////////////////////////////////////////////////////////////////////////////////////////
//Status-Register

struct stk_status
{
	unsigned signedOn:				1; //Synchronisation hergestellt
	unsigned signOnOffTrigger:		1; //Wird bei jedem Verbindungsauf- und abbau gesetzt
	unsigned connectionAssigned:	1; //ISP-Schnittstelle reserviert
	unsigned criticalFailure:		1; //Schnittstelle getrennt (Fehlermeldung)
	unsigned updateClock:			1; //Taktgeber aktualisieren

	unsigned extendedAddressing:	1; //Intern, 64k Adressierung aktiv
	unsigned reloadAddress:			1; //Intern, neue Adress bertragen
	unsigned byteAddressing:		1; //Intern, Speicher wird byteweise adressiert
	unsigned disablePolling:		1; //Intern, Schreibzugriff ber Polling nicht mglich
};

//Globales Statusregister
extern struct stk_status stk_status;

////////////////////////////////////////////////////////////////////////////////////////////
//Datenstruktur & globale Speicher

struct stk_frame
{
	uint8_t 	sequence;
	uint16_t 	dataLength;
	uint8_t		checksum;
	uint8_t		data[STK_MSG_BUFFER];
};

//Globaler Speicher fr einen Frame (senden und empfangen)
extern volatile struct stk_frame stk_frame;

////////////////////////////////////////////////////////////////////////////////////////////
//Funktionen

//Initialisiert die UART-Schnittstelle und die Protokollverarbeitung
extern void stk_init(void);

//Cronjob (Frame-Verarbeitung)
//Diese Funktion muss periodisch aufgerufen werden
//Achtung: Nicht in einem Interrupt aufrufen!
extern void stk_cronjob(void);

//Zielsystem-Spannung zuweisen fr Statusmeldung an den PC
//Einheit: 1/10 Volt (42 = 4.2V)
extern void stk_assignVTarget(uint8_t voltage);

//Verbindung trennen, Ports freigeben
extern void stk_disconnect(void);

//Taktgeber-Einstellung
extern uint8_t stk_getClockPrescaler(void);
extern uint8_t stk_getClockMatch(void);

#endif //_STK500_H_
