/*#######################################################################################

Copyright (C) 2013 Leo-Andres Hofmann
www.leo-andres.de

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.

#######################################################################################*/

#include <avr/io.h>

#include "stk500.h"
#include "ports.h"
#include "delay.h"

////////////////////////////////////////////////////////////////////////////////////////////
//Interne Variablen

static uint8_t stkT_clkDuration = 0;

////////////////////////////////////////////////////////////////////////////////////////////
//Funktionen

void stkT_ISP_setClkDuration(uint8_t duration)
{
	if( duration > 2 )
		duration = 2;

	stkT_clkDuration = duration;

	//SPI Master konfigurieren
	if( stk_status.connectionAssigned == TRUE )
	{
		switch( stkT_clkDuration )
		{
			default:
			case 2:
				STK_ISP_CLK_2;
				break;

			case 1:
				STK_ISP_CLK_1;
				break;

			case 0:
				STK_ISP_CLK_0;
				break;
		};
	}
}

/******************************************************************************************/

inline uint8_t stkT_ISP_getClkDuration(void)
{
	return stkT_clkDuration;
}

/******************************************************************************************/

uint8_t stkT_ISP_transfer_8(uint8_t data)
{
	STK_ISP_DATA = data;

	while( STK_ISP_WAIT_DATA )
		nop();

	return STK_ISP_DATA;
}

/******************************************************************************************/


uint8_t stkT_ISP_transfer_16(uint16_t data)
{
	stkT_ISP_transfer_8((data>>8) & 0xFF);

	return stkT_ISP_transfer_8(data & 0xFF);
}

/******************************************************************************************/

uint8_t stkT_ISP_transfer_32(uint32_t data)
{
	stkT_ISP_transfer_8((data>>24) & 0xFF);
	stkT_ISP_transfer_8((data>>16) & 0xFF);
	stkT_ISP_transfer_8((data>>8) & 0xFF);

	return stkT_ISP_transfer_8(data & 0xFF);
}

/******************************************************************************************/

void stkT_ISP_clockPulse(void)
{
	STK_ISP_SUSPEND;

	//Taktimpuls erzeugen
	ISP_PORT |= (1<<ISP_SCK_PIN);
	delay_us(50);
	ISP_PORT &= ~(1<<ISP_SCK_PIN);
	delay_us(25);

	STK_ISP_RESUME;
}

/******************************************************************************************/

uint8_t stkT_ISP_TestRDYBSY(uint16_t cycles)
{
	//Prfen ob RDY/BSY 1 zurckgibt (1=Busy)
	while( (stkT_ISP_transfer_32(STK_RDYBSY_ISP_CMD) & 1) && cycles )
		cycles--;

	return cycles;
}

/******************************************************************************************/

inline void stkT_ISP_assign(void)
{
	stk_status.connectionAssigned = TRUE;

	//Programmierpins reservieren
	//Reset, SCK und MOSI Ausgang, MISO Eingang
	//Reset High, SCK und MOSI Low, MISO Pullup an (0xFF zurcklesen, wenn kein Chip angeschlossen)
	ISP_PORT |= (1<<ISP_RESET_PIN);

	DDR(ISP_PORT) |= (1<<ISP_RESET_PIN) | (1<<ISP_SCK_PIN) | (1<<ISP_MOSI_PIN);
	DDR(ISP_PORT) &= ~(1<<ISP_MISO_PIN);

	ISP_PORT &= ~( (1<<ISP_SCK_PIN) | (1<<ISP_MOSI_PIN) );
	ISP_PORT |= (1<<ISP_RESET_PIN) | (1<<ISP_MISO_PIN);

	//SPI Master konfigurieren
	STK_ISP_ENABLE;
	stkT_ISP_setClkDuration(stkT_clkDuration);
}

/******************************************************************************************/

inline void stkT_ISP_release(void)
{
	stk_status.connectionAssigned = FALSE;

	//SPI Master deaktivieren
	STK_ISP_DISABLE; 

	//Alle Pins als Eingang konfigurieren, Pullups deaktivieren
	DDR(ISP_PORT) &= ~( (1<<ISP_RESET_PIN) | (1<<ISP_MISO_PIN) | (1<<ISP_MOSI_PIN) | (1<<ISP_SCK_PIN) ); 	
	ISP_PORT &= ~( (1<<ISP_RESET_PIN) | (1<<ISP_MISO_PIN) | (1<<ISP_MOSI_PIN) | (1<<ISP_SCK_PIN) );
}

/******************************************************************************************/

inline void stkT_ISP_tgRun(void)
{
	//Zielsystem frei laufen lassen (Reset tri-state)
	DDR(ISP_PORT) &= ~(1<<ISP_RESET_PIN);
	ISP_PORT &= ~(1<<ISP_RESET_PIN);
}

/******************************************************************************************/

inline void stkT_ISP_tgForceRun(void)
{
	//Zielsystem starten (Reset HIGH)
	ISP_PORT |= (1<<ISP_RESET_PIN);
	DDR(ISP_PORT) |= (1<<ISP_RESET_PIN);
}

/******************************************************************************************/

inline void stkT_ISP_tgStop(void)
{
	//Zielsystem anhalten (Reset LOW)
	ISP_PORT &= ~(1<<ISP_RESET_PIN);
	DDR(ISP_PORT) |= (1<<ISP_RESET_PIN);	
}

/******************************************************************************************/

inline void stkT_ISP_tgResetPulse(void)
{
	stkT_ISP_tgStop();

#if RESET_PULSE_LENGTH > 0
	delay_ms(RESET_PULSE_LENGTH);
#else
	nop();
#endif

	stkT_ISP_tgRun();
}

/******************************************************************************************/
