/*#######################################################################################

UART-Schnittstelle mit Puffer fr STK-Frames

"uart_": Allgemeine Funktionen
"uartIf_": Hardware-spezifische Funktionen

Copyright (C) 2013 Leo-Andres Hofmann
www.leo-andres.de

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.

#######################################################################################*/

#ifndef _UART_H_
	#define _UART_H_

#include "general.h"
#include "settings.h"

////////////////////////////////////////////////////////////////////////////////////////////
//Einstellungen

//Baudrate
#ifndef UART_BAUDRATE
	#define UART_IF_BAUDRATE	115200UL
#else
	#define UART_IF_BAUDRATE	UART_BAUDRATE
#endif

////////////////////////////////////////////////////////////////////////////////////////////
//Register & Steuermakros

//Interruptvektor
#define UART_IF_RX_INT_VECT		USART_RX_vect		//UART-Empfang vollstndig
#define UART_IF_TX_INT_VECT		USART_UDRE_vect		//Senderegister leer

//Baudrate
#define UART_IF_UBRRH			UBRR0H
#define UART_IF_UBRRL			UBRR0L

//Datenregister
#define UART_IF_DATA_REGISTER	UDR0

//Init-Sequenz
//Datenformat 8E1, Senden und Empfangen, Interrupt bei Datenempfang
#define UART_IF_SET_FORMAT 		UCSR0C = (1<<UCSZ01) | (1<<UCSZ00)
#define UART_IF_SET_CONTROL		UCSR0B = (1<<RXCIE0) | (1<<RXEN0) | (1<<TXEN0)

//Sendeinterrupt an/aus
#define UART_IF_TX_INT_ON		UCSR0B |= (1<<UDRIE0)
#define UART_IF_TX_INT_OFF		UCSR0B &= ~(1<<UDRIE0)

//Empfang auf Fehler prfen (Framing, Overrun, Parity)
#define UART_IF_GET_RX_STAT		(UCSR0A & ( (1<<FE0)|(1<<DOR0)|(1<<UPE0) ))

////////////////////////////////////////////////////////////////////////////////////////////
//Status-Register

struct uart_status
{
	unsigned rxInFrame:				1;	//Frame-Empfang aktiv

	unsigned txInFrame:				1;	//Frame-Senden aktiv
	unsigned txStart:				1;	//Startposition

#ifdef BOOTLOADER_PASSWORD
	unsigned rxPassword:			3;	//Positionszhler fr Passwort-Erkennung Bootloader
#endif

	unsigned frameComplete:			1;	//Vollstndiger Frame liegt im Puffer
	unsigned frameEmpty:			1;	//Frame wurde verarbeitet, Puffer leer
	unsigned frameError:			1;	//Allgemeine Fehler beim Frame-Empfang (Paritt, Pufferberlauf)
	unsigned frameChecksumError:	1;	//Prfsumme fehlerhaft
};

//Globales Statusregister
extern volatile struct uart_status uart_status;

////////////////////////////////////////////////////////////////////////////////////////////
//Funktionen

//Initialisiert die Schnittstelle & Protokollverarbeitung
//Nach der Initialisierung ist die Schnittstelle aktiv
extern void uart_init(void);

//Sendet den aktuellen Pufferinhalt als kompletten Frame
extern void uart_sendFrame(void);

//Setzt die Statemachine vom Empfnger zurck
extern void uart_resetReceiver(void);

//Aktiviert/deaktiviert den Interrupt zum Senden
extern void uartIf_setTxInt(uint8_t state);

#endif //_UART_H_
